# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictInt, StrictStr
from typing_extensions import Annotated
from postfinancecheckout.models.analytics_query_execution_request import AnalyticsQueryExecutionRequest
from postfinancecheckout.models.analytics_query_execution_response import AnalyticsQueryExecutionResponse
from postfinancecheckout.models.result_portion_model import ResultPortionModel
from postfinancecheckout.models.submitted_analytics_query_execution import SubmittedAnalyticsQueryExecution

from postfinancecheckout.api_client import ApiClient, RequestSerialized
from postfinancecheckout.api_response import ApiResponse
from postfinancecheckout.rest import RESTResponseType



class AnalyticsQueriesService:
    def __init__(self, configuration):
        self.api_client = ApiClient(configuration=configuration)


    @validate_call
    def delete_analytics_queries_query_external_id_query_external_id(
        self,
        query_external_id: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Cancel a query execution, identifying it by its external id.



        :param query_external_id: Identifies the query execution. (required)
        :type query_external_id: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_analytics_queries_query_external_id_query_external_id_serialize(
            query_external_id=query_external_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_analytics_queries_query_external_id_query_external_id_with_http_info(
        self,
        query_external_id: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Cancel a query execution, identifying it by its external id.



        :param query_external_id: Identifies the query execution. (required)
        :type query_external_id: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_analytics_queries_query_external_id_query_external_id_serialize(
            query_external_id=query_external_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_analytics_queries_query_external_id_query_external_id_without_preload_content(
        self,
        query_external_id: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Cancel a query execution, identifying it by its external id.



        :param query_external_id: Identifies the query execution. (required)
        :type query_external_id: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_analytics_queries_query_external_id_query_external_id_serialize(
            query_external_id=query_external_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _delete_analytics_queries_query_external_id_query_external_id_serialize(
        self,
        query_external_id,
        account,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if query_external_id is not None:
            _path_params['queryExternalId'] = query_external_id
        # process the query parameters
        # process the header parameters
        if account is not None:
            _header_params['Account'] = account
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/analytics/queries/queryExternalId/{queryExternalId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def delete_analytics_queries_query_token_query_token(
        self,
        query_token: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Cancel a query execution, identifying it by its query token.



        :param query_token: Identifies the query execution. (required)
        :type query_token: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_analytics_queries_query_token_query_token_serialize(
            query_token=query_token,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_analytics_queries_query_token_query_token_with_http_info(
        self,
        query_token: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Cancel a query execution, identifying it by its query token.



        :param query_token: Identifies the query execution. (required)
        :type query_token: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_analytics_queries_query_token_query_token_serialize(
            query_token=query_token,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_analytics_queries_query_token_query_token_without_preload_content(
        self,
        query_token: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Cancel a query execution, identifying it by its query token.



        :param query_token: Identifies the query execution. (required)
        :type query_token: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_analytics_queries_query_token_query_token_serialize(
            query_token=query_token,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _delete_analytics_queries_query_token_query_token_serialize(
        self,
        query_token,
        account,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if query_token is not None:
            _path_params['queryToken'] = query_token
        # process the query parameters
        # process the header parameters
        if account is not None:
            _header_params['Account'] = account
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/analytics/queries/queryToken/{queryToken}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def get_analytics_queries(
        self,
        offset: Annotated[StrictInt, Field(description="A cursor for pagination, specifies the number of query executions to skip.")],
        limit: Annotated[StrictInt, Field(description="A limit on the number of query executions to be returned, between 1 and 100. Default is 100.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ResultPortionModel:
        """Get portion of query executions for account



        :param offset: A cursor for pagination, specifies the number of query executions to skip. (required)
        :type offset: int
        :param limit: A limit on the number of query executions to be returned, between 1 and 100. Default is 100. (required)
        :type limit: int
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_serialize(
            offset=offset,
            limit=limit,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ResultPortionModel",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_analytics_queries_with_http_info(
        self,
        offset: Annotated[StrictInt, Field(description="A cursor for pagination, specifies the number of query executions to skip.")],
        limit: Annotated[StrictInt, Field(description="A limit on the number of query executions to be returned, between 1 and 100. Default is 100.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ResultPortionModel]:
        """Get portion of query executions for account



        :param offset: A cursor for pagination, specifies the number of query executions to skip. (required)
        :type offset: int
        :param limit: A limit on the number of query executions to be returned, between 1 and 100. Default is 100. (required)
        :type limit: int
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_serialize(
            offset=offset,
            limit=limit,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ResultPortionModel",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_analytics_queries_without_preload_content(
        self,
        offset: Annotated[StrictInt, Field(description="A cursor for pagination, specifies the number of query executions to skip.")],
        limit: Annotated[StrictInt, Field(description="A limit on the number of query executions to be returned, between 1 and 100. Default is 100.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get portion of query executions for account



        :param offset: A cursor for pagination, specifies the number of query executions to skip. (required)
        :type offset: int
        :param limit: A limit on the number of query executions to be returned, between 1 and 100. Default is 100. (required)
        :type limit: int
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_serialize(
            offset=offset,
            limit=limit,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ResultPortionModel",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _get_analytics_queries_serialize(
        self,
        offset,
        limit,
        account,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if offset is not None:
            
            _query_params.append(('offset', offset))
            
        if limit is not None:
            
            _query_params.append(('limit', limit))
            
        # process the header parameters
        if account is not None:
            _header_params['Account'] = account
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/analytics/queries',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def get_analytics_queries_query_external_id_query_external_id(
        self,
        query_external_id: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SubmittedAnalyticsQueryExecution:
        """Retrieve a query execution information by its external id

        Queries are processed asynchronously and may take several minutes to complete. Avoid frequent requests, as they will not speed up processing.
        
        (The read time out for this request is 97 seconds.)

        :param query_external_id: Identifies the query execution. (required)
        :type query_external_id: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_query_external_id_query_external_id_serialize(
            query_external_id=query_external_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SubmittedAnalyticsQueryExecution",
            '202': "SubmittedAnalyticsQueryExecution",
            '404': "RestApiErrorResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = 97
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_analytics_queries_query_external_id_query_external_id_with_http_info(
        self,
        query_external_id: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SubmittedAnalyticsQueryExecution]:
        """Retrieve a query execution information by its external id

        Queries are processed asynchronously and may take several minutes to complete. Avoid frequent requests, as they will not speed up processing.
        
        (The read time out for this request is 97 seconds.)

        :param query_external_id: Identifies the query execution. (required)
        :type query_external_id: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_query_external_id_query_external_id_serialize(
            query_external_id=query_external_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SubmittedAnalyticsQueryExecution",
            '202': "SubmittedAnalyticsQueryExecution",
            '404': "RestApiErrorResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = 97
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_analytics_queries_query_external_id_query_external_id_without_preload_content(
        self,
        query_external_id: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve a query execution information by its external id

        Queries are processed asynchronously and may take several minutes to complete. Avoid frequent requests, as they will not speed up processing.
        
        (The read time out for this request is 97 seconds.)

        :param query_external_id: Identifies the query execution. (required)
        :type query_external_id: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_query_external_id_query_external_id_serialize(
            query_external_id=query_external_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SubmittedAnalyticsQueryExecution",
            '202': "SubmittedAnalyticsQueryExecution",
            '404': "RestApiErrorResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = 97
        )
        return response_data.response


    def _get_analytics_queries_query_external_id_query_external_id_serialize(
        self,
        query_external_id,
        account,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if query_external_id is not None:
            _path_params['queryExternalId'] = query_external_id
        # process the query parameters
        # process the header parameters
        if account is not None:
            _header_params['Account'] = account
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/analytics/queries/queryExternalId/{queryExternalId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def get_analytics_queries_query_external_id_query_external_id_result(
        self,
        query_external_id: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> str:
        """Generate a temporary URL to download the query result. It retrieves the query by its external id

        Generate a short-lived (5-minute) URL for downloading the Analytics query result file. Note that each URL generation is counted as a potential download and will be billed accordingly. Do not use this endpoint for periodic checks of file availability. Instead, use the 'Retrieve a query execution' endpoint for status checks.
        

        :param query_external_id: Identifies the query execution. (required)
        :type query_external_id: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_query_external_id_query_external_id_result_serialize(
            query_external_id=query_external_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "str",
            '202': None,
            '204': None,
            '404': "RestApiErrorResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_analytics_queries_query_external_id_query_external_id_result_with_http_info(
        self,
        query_external_id: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[str]:
        """Generate a temporary URL to download the query result. It retrieves the query by its external id

        Generate a short-lived (5-minute) URL for downloading the Analytics query result file. Note that each URL generation is counted as a potential download and will be billed accordingly. Do not use this endpoint for periodic checks of file availability. Instead, use the 'Retrieve a query execution' endpoint for status checks.
        

        :param query_external_id: Identifies the query execution. (required)
        :type query_external_id: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_query_external_id_query_external_id_result_serialize(
            query_external_id=query_external_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "str",
            '202': None,
            '204': None,
            '404': "RestApiErrorResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_analytics_queries_query_external_id_query_external_id_result_without_preload_content(
        self,
        query_external_id: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Generate a temporary URL to download the query result. It retrieves the query by its external id

        Generate a short-lived (5-minute) URL for downloading the Analytics query result file. Note that each URL generation is counted as a potential download and will be billed accordingly. Do not use this endpoint for periodic checks of file availability. Instead, use the 'Retrieve a query execution' endpoint for status checks.
        

        :param query_external_id: Identifies the query execution. (required)
        :type query_external_id: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_query_external_id_query_external_id_result_serialize(
            query_external_id=query_external_id,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "str",
            '202': None,
            '204': None,
            '404': "RestApiErrorResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _get_analytics_queries_query_external_id_query_external_id_result_serialize(
        self,
        query_external_id,
        account,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if query_external_id is not None:
            _path_params['queryExternalId'] = query_external_id
        # process the query parameters
        # process the header parameters
        if account is not None:
            _header_params['Account'] = account
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/analytics/queries/queryExternalId/{queryExternalId}/result',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def get_analytics_queries_query_token_query_token(
        self,
        query_token: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SubmittedAnalyticsQueryExecution:
        """Retrieve a query execution information by its query token

        Queries are processed asynchronously and may take several minutes to complete. Avoid frequent requests, as they will not speed up processing.
        
        (The read time out for this request is 97 seconds.)

        :param query_token: Identifies the query execution. (required)
        :type query_token: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_query_token_query_token_serialize(
            query_token=query_token,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SubmittedAnalyticsQueryExecution",
            '202': "SubmittedAnalyticsQueryExecution",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = 97
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_analytics_queries_query_token_query_token_with_http_info(
        self,
        query_token: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SubmittedAnalyticsQueryExecution]:
        """Retrieve a query execution information by its query token

        Queries are processed asynchronously and may take several minutes to complete. Avoid frequent requests, as they will not speed up processing.
        
        (The read time out for this request is 97 seconds.)

        :param query_token: Identifies the query execution. (required)
        :type query_token: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_query_token_query_token_serialize(
            query_token=query_token,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SubmittedAnalyticsQueryExecution",
            '202': "SubmittedAnalyticsQueryExecution",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = 97
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_analytics_queries_query_token_query_token_without_preload_content(
        self,
        query_token: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve a query execution information by its query token

        Queries are processed asynchronously and may take several minutes to complete. Avoid frequent requests, as they will not speed up processing.
        
        (The read time out for this request is 97 seconds.)

        :param query_token: Identifies the query execution. (required)
        :type query_token: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_query_token_query_token_serialize(
            query_token=query_token,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SubmittedAnalyticsQueryExecution",
            '202': "SubmittedAnalyticsQueryExecution",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = 97
        )
        return response_data.response


    def _get_analytics_queries_query_token_query_token_serialize(
        self,
        query_token,
        account,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if query_token is not None:
            _path_params['queryToken'] = query_token
        # process the query parameters
        # process the header parameters
        if account is not None:
            _header_params['Account'] = account
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/analytics/queries/queryToken/{queryToken}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def get_analytics_queries_query_token_query_token_result(
        self,
        query_token: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> str:
        """Generate a temporary URL to download the query result. It retrieves the query by its query token

        Generate a short-lived (5-minute) URL for downloading the Analytics query result file. Note that each URL generation is counted as a potential download and will be billed accordingly. Do not use this endpoint for periodic checks of file availability. Instead, use the 'Retrieve a query execution' endpoint for status checks.
        

        :param query_token: Identifies the query execution. (required)
        :type query_token: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_query_token_query_token_result_serialize(
            query_token=query_token,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "str",
            '202': None,
            '204': None,
            '404': "RestApiErrorResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_analytics_queries_query_token_query_token_result_with_http_info(
        self,
        query_token: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[str]:
        """Generate a temporary URL to download the query result. It retrieves the query by its query token

        Generate a short-lived (5-minute) URL for downloading the Analytics query result file. Note that each URL generation is counted as a potential download and will be billed accordingly. Do not use this endpoint for periodic checks of file availability. Instead, use the 'Retrieve a query execution' endpoint for status checks.
        

        :param query_token: Identifies the query execution. (required)
        :type query_token: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_query_token_query_token_result_serialize(
            query_token=query_token,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "str",
            '202': None,
            '204': None,
            '404': "RestApiErrorResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_analytics_queries_query_token_query_token_result_without_preload_content(
        self,
        query_token: Annotated[StrictStr, Field(description="Identifies the query execution.")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Generate a temporary URL to download the query result. It retrieves the query by its query token

        Generate a short-lived (5-minute) URL for downloading the Analytics query result file. Note that each URL generation is counted as a potential download and will be billed accordingly. Do not use this endpoint for periodic checks of file availability. Instead, use the 'Retrieve a query execution' endpoint for status checks.
        

        :param query_token: Identifies the query execution. (required)
        :type query_token: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_analytics_queries_query_token_query_token_result_serialize(
            query_token=query_token,
            account=account,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "str",
            '202': None,
            '204': None,
            '404': "RestApiErrorResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _get_analytics_queries_query_token_query_token_result_serialize(
        self,
        query_token,
        account,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if query_token is not None:
            _path_params['queryToken'] = query_token
        # process the query parameters
        # process the header parameters
        if account is not None:
            _header_params['Account'] = account
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/analytics/queries/queryToken/{queryToken}/result',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def post_analytics_queries_submit(
        self,
        query_external_id: Annotated[StrictStr, Field(description="A unique id to be provided for each query. The same id for different queries will be only executed the first time")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        analytics_query_execution_request: AnalyticsQueryExecutionRequest,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> AnalyticsQueryExecutionResponse:
        """Submit a query execution



        :param query_external_id: A unique id to be provided for each query. The same id for different queries will be only executed the first time (required)
        :type query_external_id: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param analytics_query_execution_request: (required)
        :type analytics_query_execution_request: AnalyticsQueryExecutionRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_analytics_queries_submit_serialize(
            query_external_id=query_external_id,
            account=account,
            analytics_query_execution_request=analytics_query_execution_request,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "AnalyticsQueryExecutionResponse",
            '409': "RestApiErrorResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def post_analytics_queries_submit_with_http_info(
        self,
        query_external_id: Annotated[StrictStr, Field(description="A unique id to be provided for each query. The same id for different queries will be only executed the first time")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        analytics_query_execution_request: AnalyticsQueryExecutionRequest,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[AnalyticsQueryExecutionResponse]:
        """Submit a query execution



        :param query_external_id: A unique id to be provided for each query. The same id for different queries will be only executed the first time (required)
        :type query_external_id: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param analytics_query_execution_request: (required)
        :type analytics_query_execution_request: AnalyticsQueryExecutionRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_analytics_queries_submit_serialize(
            query_external_id=query_external_id,
            account=account,
            analytics_query_execution_request=analytics_query_execution_request,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "AnalyticsQueryExecutionResponse",
            '409': "RestApiErrorResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def post_analytics_queries_submit_without_preload_content(
        self,
        query_external_id: Annotated[StrictStr, Field(description="A unique id to be provided for each query. The same id for different queries will be only executed the first time")],
        account: Annotated[StrictInt, Field(description="Specifies the ID of the account the operation should be executed in.")],
        analytics_query_execution_request: AnalyticsQueryExecutionRequest,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Submit a query execution



        :param query_external_id: A unique id to be provided for each query. The same id for different queries will be only executed the first time (required)
        :type query_external_id: str
        :param account: Specifies the ID of the account the operation should be executed in. (required)
        :type account: int
        :param analytics_query_execution_request: (required)
        :type analytics_query_execution_request: AnalyticsQueryExecutionRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_analytics_queries_submit_serialize(
            query_external_id=query_external_id,
            account=account,
            analytics_query_execution_request=analytics_query_execution_request,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "AnalyticsQueryExecutionResponse",
            '409': "RestApiErrorResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _post_analytics_queries_submit_serialize(
        self,
        query_external_id,
        account,
        analytics_query_execution_request,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if query_external_id is not None:
            
            _query_params.append(('queryExternalId', query_external_id))
            
        # process the header parameters
        if account is not None:
            _header_params['Account'] = account
        # process the form parameters
        # process the body parameter
        if analytics_query_execution_request is not None:
            _body_params = analytics_query_execution_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/analytics/queries/submit',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )






# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from postfinancecheckout.models.failure_reason import FailureReason
from postfinancecheckout.models.label import Label
from postfinancecheckout.models.line_item import LineItem
from postfinancecheckout.models.transaction_completion_mode import TransactionCompletionMode
from postfinancecheckout.models.transaction_completion_state import TransactionCompletionState
from postfinancecheckout.models.transaction_line_item_version import TransactionLineItemVersion
from typing import Optional, Set
from typing_extensions import Self

class TransactionCompletion(BaseModel):
    """
    TransactionCompletion
    """ # noqa: E501
    line_item_version: Optional[TransactionLineItemVersion] = Field(default=None, alias="lineItemVersion")
    statement_descriptor: Optional[Annotated[str, Field(strict=True, max_length=80)]] = Field(default=None, description="The statement descriptor that appears on a customer's bank statement, providing an explanation for charges or payments, helping customers identify the transaction.", alias="statementDescriptor")
    base_line_items: Optional[List[LineItem]] = Field(default=None, description="The original line items from the transaction that serve as the baseline for this completion.", alias="baseLineItems")
    processing_on: Optional[datetime] = Field(default=None, description="The date and time when the processing of the transaction completion was started.", alias="processingOn")
    invoice_merchant_reference: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The merchant's reference used to identify the invoice.", alias="invoiceMerchantReference")
    language: Optional[StrictStr] = Field(default=None, description="The language that is linked to the object.")
    remaining_line_items: Optional[List[LineItem]] = Field(default=None, description="The line items yet to be captured in the transaction.", alias="remainingLineItems")
    created_on: Optional[datetime] = Field(default=None, description="The date and time when the object was created.", alias="createdOn")
    line_items: Optional[List[LineItem]] = Field(default=None, description="The line items captured in this transaction completion.", alias="lineItems")
    mode: Optional[TransactionCompletionMode] = None
    succeeded_on: Optional[datetime] = Field(default=None, description="The date and time when the transaction completion succeeded.", alias="succeededOn")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    state: Optional[TransactionCompletionState] = None
    linked_transaction: Optional[StrictInt] = Field(default=None, description="The payment transaction this object is linked to.", alias="linkedTransaction")
    payment_information: Optional[StrictStr] = Field(default=None, description="Payment-specific details related to this transaction completion such as payment instructions or references needed for processing.", alias="paymentInformation")
    amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The total amount to be captured in this completion, including taxes.")
    last_completion: Optional[StrictBool] = Field(default=None, description="Whether this is the final completion for the transaction. After the last completion is successfully created, the transaction enters its final state, and no further completions can occur.", alias="lastCompletion")
    planned_purge_date: Optional[datetime] = Field(default=None, description="The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.", alias="plannedPurgeDate")
    external_id: Optional[Annotated[str, Field(min_length=1, strict=True, max_length=100)]] = Field(default=None, description="A client-generated nonce which uniquely identifies some action to be executed. Subsequent requests with the same external ID do not execute the action again, but return the original result.", alias="externalId")
    time_zone: Optional[StrictStr] = Field(default=None, description="The time zone that this object is associated with.", alias="timeZone")
    space_view_id: Optional[StrictInt] = Field(default=None, description="The ID of the space view this object is linked to.", alias="spaceViewId")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    labels: Optional[List[Label]] = Field(default=None, description="The labels providing additional information about the object.")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    timeout_on: Optional[datetime] = Field(default=None, description="The date and time when the object will expire.", alias="timeoutOn")
    created_by: Optional[StrictInt] = Field(default=None, description="The ID of the user the transaction completion was created by.", alias="createdBy")
    next_update_on: Optional[datetime] = Field(default=None, description="The date and time when the next update of the object's state is planned.", alias="nextUpdateOn")
    failure_reason: Optional[FailureReason] = Field(default=None, alias="failureReason")
    tax_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The portion of the captured amount that corresponds to taxes.", alias="taxAmount")
    failed_on: Optional[datetime] = Field(default=None, description="The date and time when the transaction completion failed.", alias="failedOn")
    processor_reference: Optional[StrictStr] = Field(default=None, description="The reference ID provided by the payment processor, used to trace the completion through the external payment system.", alias="processorReference")
    __properties: ClassVar[List[str]] = ["lineItemVersion", "statementDescriptor", "baseLineItems", "processingOn", "invoiceMerchantReference", "language", "remainingLineItems", "createdOn", "lineItems", "mode", "succeededOn", "id", "state", "linkedTransaction", "paymentInformation", "amount", "lastCompletion", "plannedPurgeDate", "externalId", "timeZone", "spaceViewId", "version", "labels", "linkedSpaceId", "timeoutOn", "createdBy", "nextUpdateOn", "failureReason", "taxAmount", "failedOn", "processorReference"]

    @field_validator('statement_descriptor')
    def statement_descriptor_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"[a-zA-Z0-9\s.,_?+\/-]*", value):
            raise ValueError(r"must validate the regular expression /[a-zA-Z0-9\s.,_?+\/-]*/")
        return value

    @field_validator('invoice_merchant_reference')
    def invoice_merchant_reference_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"[	\x20-\x7e]*", value):
            raise ValueError(r"must validate the regular expression /[	\x20-\x7e]*/")
        return value

    @field_validator('external_id')
    def external_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"[	\x20-\x7e]*", value):
            raise ValueError(r"must validate the regular expression /[	\x20-\x7e]*/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TransactionCompletion from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "statement_descriptor",
            "base_line_items",
            "processing_on",
            "invoice_merchant_reference",
            "language",
            "remaining_line_items",
            "created_on",
            "line_items",
            "succeeded_on",
            "id",
            "linked_transaction",
            "payment_information",
            "amount",
            "last_completion",
            "planned_purge_date",
            "external_id",
            "time_zone",
            "space_view_id",
            "version",
            "labels",
            "linked_space_id",
            "timeout_on",
            "created_by",
            "next_update_on",
            "tax_amount",
            "failed_on",
            "processor_reference",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of line_item_version
        if self.line_item_version:
            _dict['lineItemVersion'] = self.line_item_version.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in base_line_items (list)
        _items = []
        if self.base_line_items:
            for _item_base_line_items in self.base_line_items:
                if _item_base_line_items:
                    _items.append(_item_base_line_items.to_dict())
            _dict['baseLineItems'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in remaining_line_items (list)
        _items = []
        if self.remaining_line_items:
            for _item_remaining_line_items in self.remaining_line_items:
                if _item_remaining_line_items:
                    _items.append(_item_remaining_line_items.to_dict())
            _dict['remainingLineItems'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in line_items (list)
        _items = []
        if self.line_items:
            for _item_line_items in self.line_items:
                if _item_line_items:
                    _items.append(_item_line_items.to_dict())
            _dict['lineItems'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in labels (list)
        _items = []
        if self.labels:
            for _item_labels in self.labels:
                if _item_labels:
                    _items.append(_item_labels.to_dict())
            _dict['labels'] = _items
        # override the default output from pydantic by calling `to_dict()` of failure_reason
        if self.failure_reason:
            _dict['failureReason'] = self.failure_reason.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TransactionCompletion from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "lineItemVersion": TransactionLineItemVersion.from_dict(obj["lineItemVersion"]) if obj.get("lineItemVersion") is not None else None,
            "statementDescriptor": obj.get("statementDescriptor"),
            "baseLineItems": [LineItem.from_dict(_item) for _item in obj["baseLineItems"]] if obj.get("baseLineItems") is not None else None,
            "processingOn": obj.get("processingOn"),
            "invoiceMerchantReference": obj.get("invoiceMerchantReference"),
            "language": obj.get("language"),
            "remainingLineItems": [LineItem.from_dict(_item) for _item in obj["remainingLineItems"]] if obj.get("remainingLineItems") is not None else None,
            "createdOn": obj.get("createdOn"),
            "lineItems": [LineItem.from_dict(_item) for _item in obj["lineItems"]] if obj.get("lineItems") is not None else None,
            "mode": obj.get("mode"),
            "succeededOn": obj.get("succeededOn"),
            "id": obj.get("id"),
            "state": obj.get("state"),
            "linkedTransaction": obj.get("linkedTransaction"),
            "paymentInformation": obj.get("paymentInformation"),
            "amount": obj.get("amount"),
            "lastCompletion": obj.get("lastCompletion"),
            "plannedPurgeDate": obj.get("plannedPurgeDate"),
            "externalId": obj.get("externalId"),
            "timeZone": obj.get("timeZone"),
            "spaceViewId": obj.get("spaceViewId"),
            "version": obj.get("version"),
            "labels": [Label.from_dict(_item) for _item in obj["labels"]] if obj.get("labels") is not None else None,
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "timeoutOn": obj.get("timeoutOn"),
            "createdBy": obj.get("createdBy"),
            "nextUpdateOn": obj.get("nextUpdateOn"),
            "failureReason": FailureReason.from_dict(obj["failureReason"]) if obj.get("failureReason") is not None else None,
            "taxAmount": obj.get("taxAmount"),
            "failedOn": obj.get("failedOn"),
            "processorReference": obj.get("processorReference")
        })
        return _obj





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from postfinancecheckout.models.card_cryptogram_create import CardCryptogramCreate
from postfinancecheckout.models.cardholder_authentication_create import CardholderAuthenticationCreate
from postfinancecheckout.models.pan_type import PanType
from postfinancecheckout.models.recurring_indicator import RecurringIndicator
from typing import Optional, Set
from typing_extensions import Self

class AuthenticatedCardDataCreate(BaseModel):
    """
    AuthenticatedCardDataCreate
    """ # noqa: E501
    expiry_date: Optional[Annotated[str, Field(strict=True)]] = Field(default=None, description="The expiry date of the card, indicating its validity period in yyyy-mm format (e.g., 2023-09).", alias="expiryDate")
    pan_type: Optional[PanType] = Field(default=None, alias="panType")
    card_holder_name: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The name of the cardholder, as printed on the card, identifying the card owner.", alias="cardHolderName")
    card_verification_code: Optional[Annotated[str, Field(min_length=3, strict=True, max_length=4)]] = Field(default=None, description="The security code used to validate the card during transactions.", alias="cardVerificationCode")
    primary_account_number: Annotated[str, Field(min_length=10, strict=True, max_length=30)] = Field(description="The card's primary account number (PAN), the unique identifier of the card.", alias="primaryAccountNumber")
    recurring_indicator: Optional[RecurringIndicator] = Field(default=None, alias="recurringIndicator")
    scheme_transaction_reference: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="A reference specific to the card's transaction within its payment scheme.", alias="schemeTransactionReference")
    cardholder_authentication: Optional[CardholderAuthenticationCreate] = Field(default=None, alias="cardholderAuthentication")
    token_requestor_id: Optional[StrictStr] = Field(default=None, description="The token requestor identifier (TRID) identifies the entity requesting tokenization for a card transaction.", alias="tokenRequestorId")
    cryptogram: Optional[CardCryptogramCreate] = None
    __properties: ClassVar[List[str]] = ["expiryDate", "panType", "cardHolderName", "cardVerificationCode", "primaryAccountNumber", "recurringIndicator", "schemeTransactionReference", "cardholderAuthentication", "tokenRequestorId", "cryptogram"]

    @field_validator('expiry_date')
    def expiry_date_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"(\d{4})-(11|12|10|0[1-9])", value):
            raise ValueError(r"must validate the regular expression /(\d{4})-(11|12|10|0[1-9])/")
        return value

    @field_validator('card_verification_code')
    def card_verification_code_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"([0-9 ]+)", value):
            raise ValueError(r"must validate the regular expression /([0-9 ]+)/")
        return value

    @field_validator('primary_account_number')
    def primary_account_number_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"([0-9 ]+)", value):
            raise ValueError(r"must validate the regular expression /([0-9 ]+)/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AuthenticatedCardDataCreate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of cardholder_authentication
        if self.cardholder_authentication:
            _dict['cardholderAuthentication'] = self.cardholder_authentication.to_dict()
        # override the default output from pydantic by calling `to_dict()` of cryptogram
        if self.cryptogram:
            _dict['cryptogram'] = self.cryptogram.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AuthenticatedCardDataCreate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "expiryDate": obj.get("expiryDate"),
            "panType": obj.get("panType"),
            "cardHolderName": obj.get("cardHolderName"),
            "cardVerificationCode": obj.get("cardVerificationCode"),
            "primaryAccountNumber": obj.get("primaryAccountNumber"),
            "recurringIndicator": obj.get("recurringIndicator"),
            "schemeTransactionReference": obj.get("schemeTransactionReference"),
            "cardholderAuthentication": CardholderAuthenticationCreate.from_dict(obj["cardholderAuthentication"]) if obj.get("cardholderAuthentication") is not None else None,
            "tokenRequestorId": obj.get("tokenRequestorId"),
            "cryptogram": CardCryptogramCreate.from_dict(obj["cryptogram"]) if obj.get("cryptogram") is not None else None
        })
        return _obj





# coding: utf-8

# flake8: noqa

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


__version__ = "6.3.0"

# import apis into sdk package
from postfinancecheckout.service.accounts_service import AccountsService
from postfinancecheckout.service.analytics_queries_service import AnalyticsQueriesService
from postfinancecheckout.service.application_users_service import ApplicationUsersService
from postfinancecheckout.service.application_users_roles_service import ApplicationUsersRolesService
from postfinancecheckout.service.bank_accounts_service import BankAccountsService
from postfinancecheckout.service.bank_transactions_service import BankTransactionsService
from postfinancecheckout.service.charge_attempts_service import ChargeAttemptsService
from postfinancecheckout.service.charge_bank_transactions_service import ChargeBankTransactionsService
from postfinancecheckout.service.charge_flow_levels_service import ChargeFlowLevelsService
from postfinancecheckout.service.charge_flows_service import ChargeFlowsService
from postfinancecheckout.service.charge_flows_level_payment_links_service import ChargeFlowsLevelPaymentLinksService
from postfinancecheckout.service.condition_types_service import ConditionTypesService
from postfinancecheckout.service.countries_service import CountriesService
from postfinancecheckout.service.currencies_service import CurrenciesService
from postfinancecheckout.service.currency_bank_accounts_service import CurrencyBankAccountsService
from postfinancecheckout.service.customer_addresses_service import CustomerAddressesService
from postfinancecheckout.service.customer_comments_service import CustomerCommentsService
from postfinancecheckout.service.customers_service import CustomersService
from postfinancecheckout.service.delivery_indications_service import DeliveryIndicationsService
from postfinancecheckout.service.document_template_types_service import DocumentTemplateTypesService
from postfinancecheckout.service.document_templates_service import DocumentTemplatesService
from postfinancecheckout.service.dunning_cases_service import DunningCasesService
from postfinancecheckout.service.dunning_flow_levels_service import DunningFlowLevelsService
from postfinancecheckout.service.dunning_flows_service import DunningFlowsService
from postfinancecheckout.service.express_checkout_service import ExpressCheckoutService
from postfinancecheckout.service.external_transfer_bank_transactions_service import ExternalTransferBankTransactionsService
from postfinancecheckout.service.human_users_service import HumanUsersService
from postfinancecheckout.service.human_users_roles_service import HumanUsersRolesService
from postfinancecheckout.service.internal_transfer_bank_transactions_service import InternalTransferBankTransactionsService
from postfinancecheckout.service.label_descriptors_service import LabelDescriptorsService
from postfinancecheckout.service.languages_service import LanguagesService
from postfinancecheckout.service.legal_organization_forms_service import LegalOrganizationFormsService
from postfinancecheckout.service.manual_tasks_service import ManualTasksService
from postfinancecheckout.service.payment_connector_configurations_service import PaymentConnectorConfigurationsService
from postfinancecheckout.service.payment_connectors_service import PaymentConnectorsService
from postfinancecheckout.service.payment_links_service import PaymentLinksService
from postfinancecheckout.service.payment_method_brands_service import PaymentMethodBrandsService
from postfinancecheckout.service.payment_method_configurations_service import PaymentMethodConfigurationsService
from postfinancecheckout.service.payment_methods_service import PaymentMethodsService
from postfinancecheckout.service.payment_processor_configurations_service import PaymentProcessorConfigurationsService
from postfinancecheckout.service.payment_processors_service import PaymentProcessorsService
from postfinancecheckout.service.payment_sales_channels_service import PaymentSalesChannelsService
from postfinancecheckout.service.payment_terminal_transaction_summaries_service import PaymentTerminalTransactionSummariesService
from postfinancecheckout.service.payment_terminals_service import PaymentTerminalsService
from postfinancecheckout.service.permissions_service import PermissionsService
from postfinancecheckout.service.refund_bank_transactions_service import RefundBankTransactionsService
from postfinancecheckout.service.refund_comments_service import RefundCommentsService
from postfinancecheckout.service.refund_recovery_bank_transactions_service import RefundRecoveryBankTransactionsService
from postfinancecheckout.service.refunds_service import RefundsService
from postfinancecheckout.service.roles_service import RolesService
from postfinancecheckout.service.single_sign_on_users_service import SingleSignOnUsersService
from postfinancecheckout.service.single_sign_on_users_roles_service import SingleSignOnUsersRolesService
from postfinancecheckout.service.spaces_service import SpacesService
from postfinancecheckout.service.static_values_service import StaticValuesService
from postfinancecheckout.service.subscribers_service import SubscribersService
from postfinancecheckout.service.subscription_affiliates_service import SubscriptionAffiliatesService
from postfinancecheckout.service.subscription_charges_service import SubscriptionChargesService
from postfinancecheckout.service.subscription_ledger_entries_service import SubscriptionLedgerEntriesService
from postfinancecheckout.service.subscription_metric_usage_reports_service import SubscriptionMetricUsageReportsService
from postfinancecheckout.service.subscription_metrics_service import SubscriptionMetricsService
from postfinancecheckout.service.subscription_period_bills_service import SubscriptionPeriodBillsService
from postfinancecheckout.service.subscription_product_component_groups_service import SubscriptionProductComponentGroupsService
from postfinancecheckout.service.subscription_product_components_service import SubscriptionProductComponentsService
from postfinancecheckout.service.subscription_product_metered_fee_tiers_service import SubscriptionProductMeteredFeeTiersService
from postfinancecheckout.service.subscription_product_metered_fees_service import SubscriptionProductMeteredFeesService
from postfinancecheckout.service.subscription_product_period_fees_service import SubscriptionProductPeriodFeesService
from postfinancecheckout.service.subscription_product_retirements_service import SubscriptionProductRetirementsService
from postfinancecheckout.service.subscription_product_setup_fees_service import SubscriptionProductSetupFeesService
from postfinancecheckout.service.subscription_product_version_retirements_service import SubscriptionProductVersionRetirementsService
from postfinancecheckout.service.subscription_product_versions_service import SubscriptionProductVersionsService
from postfinancecheckout.service.subscription_products_service import SubscriptionProductsService
from postfinancecheckout.service.subscription_suspensions_service import SubscriptionSuspensionsService
from postfinancecheckout.service.subscription_versions_service import SubscriptionVersionsService
from postfinancecheckout.service.subscriptions_service import SubscriptionsService
from postfinancecheckout.service.token_versions_service import TokenVersionsService
from postfinancecheckout.service.tokens_service import TokensService
from postfinancecheckout.service.transaction_client_platforms_service import TransactionClientPlatformsService
from postfinancecheckout.service.transaction_comments_service import TransactionCommentsService
from postfinancecheckout.service.transaction_completions_service import TransactionCompletionsService
from postfinancecheckout.service.transaction_invoice_comments_service import TransactionInvoiceCommentsService
from postfinancecheckout.service.transaction_invoices_service import TransactionInvoicesService
from postfinancecheckout.service.transaction_line_item_versions_service import TransactionLineItemVersionsService
from postfinancecheckout.service.transaction_voids_service import TransactionVoidsService
from postfinancecheckout.service.transactions_service import TransactionsService
from postfinancecheckout.service.web_apps_service import WebAppsService
from postfinancecheckout.service.webhook_encryption_keys_service import WebhookEncryptionKeysService
from postfinancecheckout.service.webhook_listeners_service import WebhookListenersService
from postfinancecheckout.service.webhook_urls_service import WebhookURLsService

# import ApiClient
from postfinancecheckout.api_response import ApiResponse
from postfinancecheckout.api_client import ApiClient
from postfinancecheckout.configuration import Configuration
from postfinancecheckout.exceptions import OpenApiException
from postfinancecheckout.exceptions import ApiTypeError
from postfinancecheckout.exceptions import ApiValueError
from postfinancecheckout.exceptions import ApiKeyError
from postfinancecheckout.exceptions import ApiAttributeError
from postfinancecheckout.exceptions import ApiException
from postfinancecheckout.api_exception_error_codes import ApiExceptionErrorCodes
from postfinancecheckout.sdk_exception_error_codes import SdkExceptionErrorCodes
from postfinancecheckout.postfinancecheckout_sdk_exception import PostFinanceCheckoutSdkException

# import utils into sdk package
from postfinancecheckout.utils.encryption_util import EncryptionUtil

# import models into sdk package
from postfinancecheckout.models.abstract_account_update import AbstractAccountUpdate
from postfinancecheckout.models.abstract_application_user_update import AbstractApplicationUserUpdate
from postfinancecheckout.models.abstract_customer_active import AbstractCustomerActive
from postfinancecheckout.models.abstract_customer_address_active import AbstractCustomerAddressActive
from postfinancecheckout.models.abstract_customer_comment_active import AbstractCustomerCommentActive
from postfinancecheckout.models.abstract_human_user_update import AbstractHumanUserUpdate
from postfinancecheckout.models.abstract_payment_connector_configuration_update import AbstractPaymentConnectorConfigurationUpdate
from postfinancecheckout.models.abstract_payment_link_update import AbstractPaymentLinkUpdate
from postfinancecheckout.models.abstract_payment_method_configuration_update import AbstractPaymentMethodConfigurationUpdate
from postfinancecheckout.models.abstract_payment_processor_configuration_active import AbstractPaymentProcessorConfigurationActive
from postfinancecheckout.models.abstract_payment_terminal_update import AbstractPaymentTerminalUpdate
from postfinancecheckout.models.abstract_refund_comment_active import AbstractRefundCommentActive
from postfinancecheckout.models.abstract_role_update import AbstractRoleUpdate
from postfinancecheckout.models.abstract_single_sign_on_user_update import AbstractSingleSignOnUserUpdate
from postfinancecheckout.models.abstract_space_update import AbstractSpaceUpdate
from postfinancecheckout.models.abstract_subscriber_update import AbstractSubscriberUpdate
from postfinancecheckout.models.abstract_subscription_affiliate_update import AbstractSubscriptionAffiliateUpdate
from postfinancecheckout.models.abstract_subscription_metric_update import AbstractSubscriptionMetricUpdate
from postfinancecheckout.models.abstract_subscription_product_active import AbstractSubscriptionProductActive
from postfinancecheckout.models.abstract_token_update import AbstractTokenUpdate
from postfinancecheckout.models.abstract_transaction_comment_active import AbstractTransactionCommentActive
from postfinancecheckout.models.abstract_transaction_invoice_comment_active import AbstractTransactionInvoiceCommentActive
from postfinancecheckout.models.abstract_transaction_pending import AbstractTransactionPending
from postfinancecheckout.models.abstract_webhook_listener_update import AbstractWebhookListenerUpdate
from postfinancecheckout.models.abstract_webhook_url_update import AbstractWebhookUrlUpdate
from postfinancecheckout.models.account import Account
from postfinancecheckout.models.account_create import AccountCreate
from postfinancecheckout.models.account_list_response import AccountListResponse
from postfinancecheckout.models.account_search_response import AccountSearchResponse
from postfinancecheckout.models.account_state import AccountState
from postfinancecheckout.models.account_type import AccountType
from postfinancecheckout.models.account_update import AccountUpdate
from postfinancecheckout.models.address import Address
from postfinancecheckout.models.address_create import AddressCreate
from postfinancecheckout.models.analytics_query_execution_request import AnalyticsQueryExecutionRequest
from postfinancecheckout.models.analytics_query_execution_response import AnalyticsQueryExecutionResponse
from postfinancecheckout.models.application_key_state import ApplicationKeyState
from postfinancecheckout.models.application_user import ApplicationUser
from postfinancecheckout.models.application_user_create import ApplicationUserCreate
from postfinancecheckout.models.application_user_create_with_mac_key import ApplicationUserCreateWithMacKey
from postfinancecheckout.models.application_user_list_response import ApplicationUserListResponse
from postfinancecheckout.models.application_user_search_response import ApplicationUserSearchResponse
from postfinancecheckout.models.application_user_update import ApplicationUserUpdate
from postfinancecheckout.models.authenticated_card_data import AuthenticatedCardData
from postfinancecheckout.models.authenticated_card_data_create import AuthenticatedCardDataCreate
from postfinancecheckout.models.authenticated_card_request import AuthenticatedCardRequest
from postfinancecheckout.models.bank_account import BankAccount
from postfinancecheckout.models.bank_account_environment import BankAccountEnvironment
from postfinancecheckout.models.bank_account_list_response import BankAccountListResponse
from postfinancecheckout.models.bank_account_search_response import BankAccountSearchResponse
from postfinancecheckout.models.bank_account_state import BankAccountState
from postfinancecheckout.models.bank_account_type import BankAccountType
from postfinancecheckout.models.bank_transaction import BankTransaction
from postfinancecheckout.models.bank_transaction_flow_direction import BankTransactionFlowDirection
from postfinancecheckout.models.bank_transaction_list_response import BankTransactionListResponse
from postfinancecheckout.models.bank_transaction_search_response import BankTransactionSearchResponse
from postfinancecheckout.models.bank_transaction_source import BankTransactionSource
from postfinancecheckout.models.bank_transaction_state import BankTransactionState
from postfinancecheckout.models.bank_transaction_type import BankTransactionType
from postfinancecheckout.models.billing_cycle_model import BillingCycleModel
from postfinancecheckout.models.billing_cycle_type import BillingCycleType
from postfinancecheckout.models.billing_day_customization import BillingDayCustomization
from postfinancecheckout.models.card_authentication_response import CardAuthenticationResponse
from postfinancecheckout.models.card_authentication_version import CardAuthenticationVersion
from postfinancecheckout.models.card_cryptogram import CardCryptogram
from postfinancecheckout.models.card_cryptogram_create import CardCryptogramCreate
from postfinancecheckout.models.cardholder_authentication import CardholderAuthentication
from postfinancecheckout.models.cardholder_authentication_create import CardholderAuthenticationCreate
from postfinancecheckout.models.charge import Charge
from postfinancecheckout.models.charge_attempt import ChargeAttempt
from postfinancecheckout.models.charge_attempt_environment import ChargeAttemptEnvironment
from postfinancecheckout.models.charge_attempt_list_response import ChargeAttemptListResponse
from postfinancecheckout.models.charge_attempt_search_response import ChargeAttemptSearchResponse
from postfinancecheckout.models.charge_attempt_state import ChargeAttemptState
from postfinancecheckout.models.charge_bank_transaction import ChargeBankTransaction
from postfinancecheckout.models.charge_bank_transaction_list_response import ChargeBankTransactionListResponse
from postfinancecheckout.models.charge_bank_transaction_search_response import ChargeBankTransactionSearchResponse
from postfinancecheckout.models.charge_flow import ChargeFlow
from postfinancecheckout.models.charge_flow_level import ChargeFlowLevel
from postfinancecheckout.models.charge_flow_level_configuration import ChargeFlowLevelConfiguration
from postfinancecheckout.models.charge_flow_level_configuration_type import ChargeFlowLevelConfigurationType
from postfinancecheckout.models.charge_flow_level_list_response import ChargeFlowLevelListResponse
from postfinancecheckout.models.charge_flow_level_payment_link import ChargeFlowLevelPaymentLink
from postfinancecheckout.models.charge_flow_level_payment_link_list_response import ChargeFlowLevelPaymentLinkListResponse
from postfinancecheckout.models.charge_flow_level_payment_link_search_response import ChargeFlowLevelPaymentLinkSearchResponse
from postfinancecheckout.models.charge_flow_level_search_response import ChargeFlowLevelSearchResponse
from postfinancecheckout.models.charge_flow_level_state import ChargeFlowLevelState
from postfinancecheckout.models.charge_flow_list_response import ChargeFlowListResponse
from postfinancecheckout.models.charge_flow_search_response import ChargeFlowSearchResponse
from postfinancecheckout.models.charge_state import ChargeState
from postfinancecheckout.models.charge_type import ChargeType
from postfinancecheckout.models.client_platform_information_list_response import ClientPlatformInformationListResponse
from postfinancecheckout.models.client_platform_information_search_response import ClientPlatformInformationSearchResponse
from postfinancecheckout.models.completion_line_item import CompletionLineItem
from postfinancecheckout.models.completion_line_item_create import CompletionLineItemCreate
from postfinancecheckout.models.completion_list_response import CompletionListResponse
from postfinancecheckout.models.completion_search_response import CompletionSearchResponse
from postfinancecheckout.models.condition import Condition
from postfinancecheckout.models.condition_type import ConditionType
from postfinancecheckout.models.condition_type_list_response import ConditionTypeListResponse
from postfinancecheckout.models.condition_type_search_response import ConditionTypeSearchResponse
from postfinancecheckout.models.connector_invocation import ConnectorInvocation
from postfinancecheckout.models.connector_invocation_stage import ConnectorInvocationStage
from postfinancecheckout.models.country_list_response import CountryListResponse
from postfinancecheckout.models.country_search_response import CountrySearchResponse
from postfinancecheckout.models.creation_entity_state import CreationEntityState
from postfinancecheckout.models.currency_bank_account import CurrencyBankAccount
from postfinancecheckout.models.currency_bank_account_list_response import CurrencyBankAccountListResponse
from postfinancecheckout.models.currency_bank_account_search_response import CurrencyBankAccountSearchResponse
from postfinancecheckout.models.currency_list_response import CurrencyListResponse
from postfinancecheckout.models.currency_search_response import CurrencySearchResponse
from postfinancecheckout.models.customer import Customer
from postfinancecheckout.models.customer_active import CustomerActive
from postfinancecheckout.models.customer_address import CustomerAddress
from postfinancecheckout.models.customer_address_active import CustomerAddressActive
from postfinancecheckout.models.customer_address_create import CustomerAddressCreate
from postfinancecheckout.models.customer_address_list_response import CustomerAddressListResponse
from postfinancecheckout.models.customer_address_search_response import CustomerAddressSearchResponse
from postfinancecheckout.models.customer_address_type import CustomerAddressType
from postfinancecheckout.models.customer_comment import CustomerComment
from postfinancecheckout.models.customer_comment_active import CustomerCommentActive
from postfinancecheckout.models.customer_comment_create import CustomerCommentCreate
from postfinancecheckout.models.customer_comment_list_response import CustomerCommentListResponse
from postfinancecheckout.models.customer_comment_search_response import CustomerCommentSearchResponse
from postfinancecheckout.models.customer_create import CustomerCreate
from postfinancecheckout.models.customer_email_address_list_response import CustomerEmailAddressListResponse
from postfinancecheckout.models.customer_list_response import CustomerListResponse
from postfinancecheckout.models.customer_postal_address import CustomerPostalAddress
from postfinancecheckout.models.customer_postal_address_create import CustomerPostalAddressCreate
from postfinancecheckout.models.customer_search_response import CustomerSearchResponse
from postfinancecheckout.models.customers_presence import CustomersPresence
from postfinancecheckout.models.data_collection_type import DataCollectionType
from postfinancecheckout.models.delivery_indication import DeliveryIndication
from postfinancecheckout.models.delivery_indication_decision_reason import DeliveryIndicationDecisionReason
from postfinancecheckout.models.delivery_indication_list_response import DeliveryIndicationListResponse
from postfinancecheckout.models.delivery_indication_search_response import DeliveryIndicationSearchResponse
from postfinancecheckout.models.delivery_indication_state import DeliveryIndicationState
from postfinancecheckout.models.displayable_day_of_week import DisplayableDayOfWeek
from postfinancecheckout.models.displayable_month import DisplayableMonth
from postfinancecheckout.models.document_template import DocumentTemplate
from postfinancecheckout.models.document_template_list_response import DocumentTemplateListResponse
from postfinancecheckout.models.document_template_search_response import DocumentTemplateSearchResponse
from postfinancecheckout.models.document_template_type import DocumentTemplateType
from postfinancecheckout.models.document_template_type_group import DocumentTemplateTypeGroup
from postfinancecheckout.models.document_template_type_list_response import DocumentTemplateTypeListResponse
from postfinancecheckout.models.document_template_type_search_response import DocumentTemplateTypeSearchResponse
from postfinancecheckout.models.dunning_case import DunningCase
from postfinancecheckout.models.dunning_case_list_response import DunningCaseListResponse
from postfinancecheckout.models.dunning_case_search_response import DunningCaseSearchResponse
from postfinancecheckout.models.dunning_case_state import DunningCaseState
from postfinancecheckout.models.dunning_condition import DunningCondition
from postfinancecheckout.models.dunning_condition_type import DunningConditionType
from postfinancecheckout.models.dunning_flow import DunningFlow
from postfinancecheckout.models.dunning_flow_level import DunningFlowLevel
from postfinancecheckout.models.dunning_flow_level_list_response import DunningFlowLevelListResponse
from postfinancecheckout.models.dunning_flow_level_processor import DunningFlowLevelProcessor
from postfinancecheckout.models.dunning_flow_level_search_response import DunningFlowLevelSearchResponse
from postfinancecheckout.models.dunning_flow_list_response import DunningFlowListResponse
from postfinancecheckout.models.dunning_flow_search_response import DunningFlowSearchResponse
from postfinancecheckout.models.dunning_flow_type import DunningFlowType
from postfinancecheckout.models.environment import Environment
from postfinancecheckout.models.express_checkout_create_response import ExpressCheckoutCreateResponse
from postfinancecheckout.models.express_checkout_session import ExpressCheckoutSession
from postfinancecheckout.models.express_checkout_session_create import ExpressCheckoutSessionCreate
from postfinancecheckout.models.express_checkout_session_state import ExpressCheckoutSessionState
from postfinancecheckout.models.express_checkout_shipping_option import ExpressCheckoutShippingOption
from postfinancecheckout.models.express_checkout_wallet_type import ExpressCheckoutWalletType
from postfinancecheckout.models.external_transfer_bank_transaction import ExternalTransferBankTransaction
from postfinancecheckout.models.external_transfer_bank_transaction_list_response import ExternalTransferBankTransactionListResponse
from postfinancecheckout.models.external_transfer_bank_transaction_search_response import ExternalTransferBankTransactionSearchResponse
from postfinancecheckout.models.facade_user_friendly_query_status_model import FacadeUserFriendlyQueryStatusModel
from postfinancecheckout.models.failure_category import FailureCategory
from postfinancecheckout.models.failure_reason import FailureReason
from postfinancecheckout.models.feature import Feature
from postfinancecheckout.models.feature_category import FeatureCategory
from postfinancecheckout.models.gender import Gender
from postfinancecheckout.models.human_user import HumanUser
from postfinancecheckout.models.human_user_create import HumanUserCreate
from postfinancecheckout.models.human_user_list_response import HumanUserListResponse
from postfinancecheckout.models.human_user_search_response import HumanUserSearchResponse
from postfinancecheckout.models.human_user_update import HumanUserUpdate
from postfinancecheckout.models.internal_transfer_bank_transaction import InternalTransferBankTransaction
from postfinancecheckout.models.internal_transfer_bank_transaction_list_response import InternalTransferBankTransactionListResponse
from postfinancecheckout.models.internal_transfer_bank_transaction_search_response import InternalTransferBankTransactionSearchResponse
from postfinancecheckout.models.invoice_comment_list_response import InvoiceCommentListResponse
from postfinancecheckout.models.invoice_comment_search_response import InvoiceCommentSearchResponse
from postfinancecheckout.models.invoice_list_response import InvoiceListResponse
from postfinancecheckout.models.invoice_search_response import InvoiceSearchResponse
from postfinancecheckout.models.label import Label
from postfinancecheckout.models.label_descriptor import LabelDescriptor
from postfinancecheckout.models.label_descriptor_category import LabelDescriptorCategory
from postfinancecheckout.models.label_descriptor_group import LabelDescriptorGroup
from postfinancecheckout.models.label_descriptor_group_list_response import LabelDescriptorGroupListResponse
from postfinancecheckout.models.label_descriptor_group_search_response import LabelDescriptorGroupSearchResponse
from postfinancecheckout.models.label_descriptor_list_response import LabelDescriptorListResponse
from postfinancecheckout.models.label_descriptor_search_response import LabelDescriptorSearchResponse
from postfinancecheckout.models.label_descriptor_type import LabelDescriptorType
from postfinancecheckout.models.language_list_response import LanguageListResponse
from postfinancecheckout.models.language_search_response import LanguageSearchResponse
from postfinancecheckout.models.legal_organization_form import LegalOrganizationForm
from postfinancecheckout.models.legal_organization_form_list_response import LegalOrganizationFormListResponse
from postfinancecheckout.models.legal_organization_form_search_response import LegalOrganizationFormSearchResponse
from postfinancecheckout.models.line_item import LineItem
from postfinancecheckout.models.line_item_attribute import LineItemAttribute
from postfinancecheckout.models.line_item_attribute_create import LineItemAttributeCreate
from postfinancecheckout.models.line_item_create import LineItemCreate
from postfinancecheckout.models.line_item_reduction import LineItemReduction
from postfinancecheckout.models.line_item_reduction_create import LineItemReductionCreate
from postfinancecheckout.models.line_item_type import LineItemType
from postfinancecheckout.models.line_item_version_list_response import LineItemVersionListResponse
from postfinancecheckout.models.line_item_version_search_response import LineItemVersionSearchResponse
from postfinancecheckout.models.localized_string import LocalizedString
from postfinancecheckout.models.manual_task import ManualTask
from postfinancecheckout.models.manual_task_action import ManualTaskAction
from postfinancecheckout.models.manual_task_action_style import ManualTaskActionStyle
from postfinancecheckout.models.manual_task_list_response import ManualTaskListResponse
from postfinancecheckout.models.manual_task_search_response import ManualTaskSearchResponse
from postfinancecheckout.models.manual_task_state import ManualTaskState
from postfinancecheckout.models.manual_task_type import ManualTaskType
from postfinancecheckout.models.metric_list_response import MetricListResponse
from postfinancecheckout.models.metric_search_response import MetricSearchResponse
from postfinancecheckout.models.metric_usage_report_list_response import MetricUsageReportListResponse
from postfinancecheckout.models.metric_usage_report_search_response import MetricUsageReportSearchResponse
from postfinancecheckout.models.one_click_payment_mode import OneClickPaymentMode
from postfinancecheckout.models.pan_type import PanType
from postfinancecheckout.models.payment_adjustment import PaymentAdjustment
from postfinancecheckout.models.payment_adjustment_type import PaymentAdjustmentType
from postfinancecheckout.models.payment_connector import PaymentConnector
from postfinancecheckout.models.payment_connector_configuration import PaymentConnectorConfiguration
from postfinancecheckout.models.payment_connector_configuration_create import PaymentConnectorConfigurationCreate
from postfinancecheckout.models.payment_connector_configuration_list_response import PaymentConnectorConfigurationListResponse
from postfinancecheckout.models.payment_connector_configuration_search_response import PaymentConnectorConfigurationSearchResponse
from postfinancecheckout.models.payment_connector_configuration_update import PaymentConnectorConfigurationUpdate
from postfinancecheckout.models.payment_connector_feature import PaymentConnectorFeature
from postfinancecheckout.models.payment_connector_list_response import PaymentConnectorListResponse
from postfinancecheckout.models.payment_connector_search_response import PaymentConnectorSearchResponse
from postfinancecheckout.models.payment_contract import PaymentContract
from postfinancecheckout.models.payment_contract_state import PaymentContractState
from postfinancecheckout.models.payment_contract_type import PaymentContractType
from postfinancecheckout.models.payment_information_hash import PaymentInformationHash
from postfinancecheckout.models.payment_information_hash_type import PaymentInformationHashType
from postfinancecheckout.models.payment_link import PaymentLink
from postfinancecheckout.models.payment_link_active import PaymentLinkActive
from postfinancecheckout.models.payment_link_address_handling_mode import PaymentLinkAddressHandlingMode
from postfinancecheckout.models.payment_link_create import PaymentLinkCreate
from postfinancecheckout.models.payment_link_list_response import PaymentLinkListResponse
from postfinancecheckout.models.payment_link_protection_mode import PaymentLinkProtectionMode
from postfinancecheckout.models.payment_link_search_response import PaymentLinkSearchResponse
from postfinancecheckout.models.payment_link_update import PaymentLinkUpdate
from postfinancecheckout.models.payment_method import PaymentMethod
from postfinancecheckout.models.payment_method_brand import PaymentMethodBrand
from postfinancecheckout.models.payment_method_brand_list_response import PaymentMethodBrandListResponse
from postfinancecheckout.models.payment_method_brand_search_response import PaymentMethodBrandSearchResponse
from postfinancecheckout.models.payment_method_configuration import PaymentMethodConfiguration
from postfinancecheckout.models.payment_method_configuration_active import PaymentMethodConfigurationActive
from postfinancecheckout.models.payment_method_configuration_create import PaymentMethodConfigurationCreate
from postfinancecheckout.models.payment_method_configuration_list_response import PaymentMethodConfigurationListResponse
from postfinancecheckout.models.payment_method_configuration_search_response import PaymentMethodConfigurationSearchResponse
from postfinancecheckout.models.payment_method_configuration_update import PaymentMethodConfigurationUpdate
from postfinancecheckout.models.payment_method_list_response import PaymentMethodListResponse
from postfinancecheckout.models.payment_method_search_response import PaymentMethodSearchResponse
from postfinancecheckout.models.payment_primary_risk_taker import PaymentPrimaryRiskTaker
from postfinancecheckout.models.payment_processor import PaymentProcessor
from postfinancecheckout.models.payment_processor_configuration import PaymentProcessorConfiguration
from postfinancecheckout.models.payment_processor_configuration_active import PaymentProcessorConfigurationActive
from postfinancecheckout.models.payment_processor_configuration_create import PaymentProcessorConfigurationCreate
from postfinancecheckout.models.payment_processor_configuration_list_response import PaymentProcessorConfigurationListResponse
from postfinancecheckout.models.payment_processor_configuration_search_response import PaymentProcessorConfigurationSearchResponse
from postfinancecheckout.models.payment_processor_list_response import PaymentProcessorListResponse
from postfinancecheckout.models.payment_processor_search_response import PaymentProcessorSearchResponse
from postfinancecheckout.models.payment_terminal import PaymentTerminal
from postfinancecheckout.models.payment_terminal_address import PaymentTerminalAddress
from postfinancecheckout.models.payment_terminal_configuration import PaymentTerminalConfiguration
from postfinancecheckout.models.payment_terminal_configuration_state import PaymentTerminalConfigurationState
from postfinancecheckout.models.payment_terminal_configuration_version import PaymentTerminalConfigurationVersion
from postfinancecheckout.models.payment_terminal_configuration_version_state import PaymentTerminalConfigurationVersionState
from postfinancecheckout.models.payment_terminal_create import PaymentTerminalCreate
from postfinancecheckout.models.payment_terminal_dcc_transaction_sum import PaymentTerminalDccTransactionSum
from postfinancecheckout.models.payment_terminal_location import PaymentTerminalLocation
from postfinancecheckout.models.payment_terminal_location_state import PaymentTerminalLocationState
from postfinancecheckout.models.payment_terminal_location_version import PaymentTerminalLocationVersion
from postfinancecheckout.models.payment_terminal_location_version_state import PaymentTerminalLocationVersionState
from postfinancecheckout.models.payment_terminal_preparing import PaymentTerminalPreparing
from postfinancecheckout.models.payment_terminal_receipt_type import PaymentTerminalReceiptType
from postfinancecheckout.models.payment_terminal_state import PaymentTerminalState
from postfinancecheckout.models.payment_terminal_transaction_sum import PaymentTerminalTransactionSum
from postfinancecheckout.models.payment_terminal_transaction_summary import PaymentTerminalTransactionSummary
from postfinancecheckout.models.payment_terminal_transaction_summary_reference import PaymentTerminalTransactionSummaryReference
from postfinancecheckout.models.payment_terminal_type import PaymentTerminalType
from postfinancecheckout.models.payment_terminal_update import PaymentTerminalUpdate
from postfinancecheckout.models.permission import Permission
from postfinancecheckout.models.permission_list_response import PermissionListResponse
from postfinancecheckout.models.permission_search_response import PermissionSearchResponse
from postfinancecheckout.models.persistable_currency_amount import PersistableCurrencyAmount
from postfinancecheckout.models.persistable_currency_amount_update import PersistableCurrencyAmountUpdate
from postfinancecheckout.models.product_component_group_list_response import ProductComponentGroupListResponse
from postfinancecheckout.models.product_component_group_search_response import ProductComponentGroupSearchResponse
from postfinancecheckout.models.product_component_list_response import ProductComponentListResponse
from postfinancecheckout.models.product_component_search_response import ProductComponentSearchResponse
from postfinancecheckout.models.product_fee_type import ProductFeeType
from postfinancecheckout.models.product_list_response import ProductListResponse
from postfinancecheckout.models.product_metered_fee import ProductMeteredFee
from postfinancecheckout.models.product_metered_fee_list_response import ProductMeteredFeeListResponse
from postfinancecheckout.models.product_metered_fee_search_response import ProductMeteredFeeSearchResponse
from postfinancecheckout.models.product_metered_fee_tier_list_response import ProductMeteredFeeTierListResponse
from postfinancecheckout.models.product_metered_fee_tier_search_response import ProductMeteredFeeTierSearchResponse
from postfinancecheckout.models.product_metered_fee_update import ProductMeteredFeeUpdate
from postfinancecheckout.models.product_metered_tier_fee import ProductMeteredTierFee
from postfinancecheckout.models.product_metered_tier_fee_update import ProductMeteredTierFeeUpdate
from postfinancecheckout.models.product_metered_tier_pricing import ProductMeteredTierPricing
from postfinancecheckout.models.product_period_fee import ProductPeriodFee
from postfinancecheckout.models.product_period_fee_list_response import ProductPeriodFeeListResponse
from postfinancecheckout.models.product_period_fee_search_response import ProductPeriodFeeSearchResponse
from postfinancecheckout.models.product_period_fee_update import ProductPeriodFeeUpdate
from postfinancecheckout.models.product_retirement_list_response import ProductRetirementListResponse
from postfinancecheckout.models.product_retirement_search_response import ProductRetirementSearchResponse
from postfinancecheckout.models.product_search_response import ProductSearchResponse
from postfinancecheckout.models.product_setup_fee import ProductSetupFee
from postfinancecheckout.models.product_setup_fee_list_response import ProductSetupFeeListResponse
from postfinancecheckout.models.product_setup_fee_search_response import ProductSetupFeeSearchResponse
from postfinancecheckout.models.product_setup_fee_update import ProductSetupFeeUpdate
from postfinancecheckout.models.product_version_list_response import ProductVersionListResponse
from postfinancecheckout.models.product_version_retirement_list_response import ProductVersionRetirementListResponse
from postfinancecheckout.models.product_version_retirement_search_response import ProductVersionRetirementSearchResponse
from postfinancecheckout.models.product_version_search_response import ProductVersionSearchResponse
from postfinancecheckout.models.recurring_indicator import RecurringIndicator
from postfinancecheckout.models.refund import Refund
from postfinancecheckout.models.refund_bank_transaction import RefundBankTransaction
from postfinancecheckout.models.refund_bank_transaction_list_response import RefundBankTransactionListResponse
from postfinancecheckout.models.refund_bank_transaction_search_response import RefundBankTransactionSearchResponse
from postfinancecheckout.models.refund_comment import RefundComment
from postfinancecheckout.models.refund_comment_active import RefundCommentActive
from postfinancecheckout.models.refund_comment_create import RefundCommentCreate
from postfinancecheckout.models.refund_comment_list_response import RefundCommentListResponse
from postfinancecheckout.models.refund_comment_search_response import RefundCommentSearchResponse
from postfinancecheckout.models.refund_create import RefundCreate
from postfinancecheckout.models.refund_list_response import RefundListResponse
from postfinancecheckout.models.refund_recovery_bank_transaction import RefundRecoveryBankTransaction
from postfinancecheckout.models.refund_recovery_bank_transaction_list_response import RefundRecoveryBankTransactionListResponse
from postfinancecheckout.models.refund_recovery_bank_transaction_search_response import RefundRecoveryBankTransactionSearchResponse
from postfinancecheckout.models.refund_search_response import RefundSearchResponse
from postfinancecheckout.models.refund_state import RefundState
from postfinancecheckout.models.refund_type import RefundType
from postfinancecheckout.models.rendered_document import RenderedDocument
from postfinancecheckout.models.rendered_terminal_receipt import RenderedTerminalReceipt
from postfinancecheckout.models.rendered_terminal_receipt_list_response import RenderedTerminalReceiptListResponse
from postfinancecheckout.models.rendered_terminal_transaction_summary import RenderedTerminalTransactionSummary
from postfinancecheckout.models.rest_address_format import RestAddressFormat
from postfinancecheckout.models.rest_address_format_field import RestAddressFormatField
from postfinancecheckout.models.rest_api_bulk_operation_result import RestApiBulkOperationResult
from postfinancecheckout.models.rest_api_error_response import RestApiErrorResponse
from postfinancecheckout.models.rest_application_user_mac_key import RestApplicationUserMacKey
from postfinancecheckout.models.rest_application_user_mac_key_created import RestApplicationUserMacKeyCreated
from postfinancecheckout.models.rest_country import RestCountry
from postfinancecheckout.models.rest_country_state import RestCountryState
from postfinancecheckout.models.rest_currency import RestCurrency
from postfinancecheckout.models.rest_customer_email_address import RestCustomerEmailAddress
from postfinancecheckout.models.rest_language import RestLanguage
from postfinancecheckout.models.result_portion_model import ResultPortionModel
from postfinancecheckout.models.role import Role
from postfinancecheckout.models.role_create import RoleCreate
from postfinancecheckout.models.role_list_response import RoleListResponse
from postfinancecheckout.models.role_search_response import RoleSearchResponse
from postfinancecheckout.models.role_state import RoleState
from postfinancecheckout.models.role_update import RoleUpdate
from postfinancecheckout.models.sales_channel import SalesChannel
from postfinancecheckout.models.sales_channel_list_response import SalesChannelListResponse
from postfinancecheckout.models.sales_channel_search_response import SalesChannelSearchResponse
from postfinancecheckout.models.scope import Scope
from postfinancecheckout.models.scope_single_sign_on_provider import ScopeSingleSignOnProvider
from postfinancecheckout.models.single_sign_on_user import SingleSignOnUser
from postfinancecheckout.models.single_sign_on_user_create import SingleSignOnUserCreate
from postfinancecheckout.models.single_sign_on_user_list_response import SingleSignOnUserListResponse
from postfinancecheckout.models.single_sign_on_user_search_response import SingleSignOnUserSearchResponse
from postfinancecheckout.models.single_sign_on_user_update import SingleSignOnUserUpdate
from postfinancecheckout.models.sorting_order import SortingOrder
from postfinancecheckout.models.space import Space
from postfinancecheckout.models.space_address import SpaceAddress
from postfinancecheckout.models.space_address_create import SpaceAddressCreate
from postfinancecheckout.models.space_create import SpaceCreate
from postfinancecheckout.models.space_list_response import SpaceListResponse
from postfinancecheckout.models.space_search_response import SpaceSearchResponse
from postfinancecheckout.models.space_update import SpaceUpdate
from postfinancecheckout.models.space_view import SpaceView
from postfinancecheckout.models.static_value import StaticValue
from postfinancecheckout.models.static_value_list_response import StaticValueListResponse
from postfinancecheckout.models.static_value_search_response import StaticValueSearchResponse
from postfinancecheckout.models.submitted_analytics_query_execution import SubmittedAnalyticsQueryExecution
from postfinancecheckout.models.subscriber import Subscriber
from postfinancecheckout.models.subscriber_active import SubscriberActive
from postfinancecheckout.models.subscriber_create import SubscriberCreate
from postfinancecheckout.models.subscriber_list_response import SubscriberListResponse
from postfinancecheckout.models.subscriber_search_response import SubscriberSearchResponse
from postfinancecheckout.models.subscriber_update import SubscriberUpdate
from postfinancecheckout.models.subscription import Subscription
from postfinancecheckout.models.subscription_affiliate import SubscriptionAffiliate
from postfinancecheckout.models.subscription_affiliate_create import SubscriptionAffiliateCreate
from postfinancecheckout.models.subscription_affiliate_deleted import SubscriptionAffiliateDeleted
from postfinancecheckout.models.subscription_affiliate_deleting import SubscriptionAffiliateDeleting
from postfinancecheckout.models.subscription_affiliate_inactive import SubscriptionAffiliateInactive
from postfinancecheckout.models.subscription_affiliate_list_response import SubscriptionAffiliateListResponse
from postfinancecheckout.models.subscription_affiliate_search_response import SubscriptionAffiliateSearchResponse
from postfinancecheckout.models.subscription_affiliate_update import SubscriptionAffiliateUpdate
from postfinancecheckout.models.subscription_charge import SubscriptionCharge
from postfinancecheckout.models.subscription_charge_create import SubscriptionChargeCreate
from postfinancecheckout.models.subscription_charge_list_response import SubscriptionChargeListResponse
from postfinancecheckout.models.subscription_charge_processing_type import SubscriptionChargeProcessingType
from postfinancecheckout.models.subscription_charge_search_response import SubscriptionChargeSearchResponse
from postfinancecheckout.models.subscription_charge_state import SubscriptionChargeState
from postfinancecheckout.models.subscription_charge_type import SubscriptionChargeType
from postfinancecheckout.models.subscription_component_configuration import SubscriptionComponentConfiguration
from postfinancecheckout.models.subscription_component_reference_configuration import SubscriptionComponentReferenceConfiguration
from postfinancecheckout.models.subscription_create_request import SubscriptionCreateRequest
from postfinancecheckout.models.subscription_initialize_subscriber_present_request import SubscriptionInitializeSubscriberPresentRequest
from postfinancecheckout.models.subscription_ledger_entry import SubscriptionLedgerEntry
from postfinancecheckout.models.subscription_ledger_entry_create import SubscriptionLedgerEntryCreate
from postfinancecheckout.models.subscription_ledger_entry_list_response import SubscriptionLedgerEntryListResponse
from postfinancecheckout.models.subscription_ledger_entry_search_response import SubscriptionLedgerEntrySearchResponse
from postfinancecheckout.models.subscription_ledger_entry_state import SubscriptionLedgerEntryState
from postfinancecheckout.models.subscription_list_response import SubscriptionListResponse
from postfinancecheckout.models.subscription_metric import SubscriptionMetric
from postfinancecheckout.models.subscription_metric_active import SubscriptionMetricActive
from postfinancecheckout.models.subscription_metric_create import SubscriptionMetricCreate
from postfinancecheckout.models.subscription_metric_type import SubscriptionMetricType
from postfinancecheckout.models.subscription_metric_update import SubscriptionMetricUpdate
from postfinancecheckout.models.subscription_metric_usage_report import SubscriptionMetricUsageReport
from postfinancecheckout.models.subscription_metric_usage_report_create import SubscriptionMetricUsageReportCreate
from postfinancecheckout.models.subscription_pending import SubscriptionPending
from postfinancecheckout.models.subscription_period_bill import SubscriptionPeriodBill
from postfinancecheckout.models.subscription_period_bill_list_response import SubscriptionPeriodBillListResponse
from postfinancecheckout.models.subscription_period_bill_search_response import SubscriptionPeriodBillSearchResponse
from postfinancecheckout.models.subscription_period_bill_state import SubscriptionPeriodBillState
from postfinancecheckout.models.subscription_product import SubscriptionProduct
from postfinancecheckout.models.subscription_product_active import SubscriptionProductActive
from postfinancecheckout.models.subscription_product_component import SubscriptionProductComponent
from postfinancecheckout.models.subscription_product_component_group import SubscriptionProductComponentGroup
from postfinancecheckout.models.subscription_product_component_group_update import SubscriptionProductComponentGroupUpdate
from postfinancecheckout.models.subscription_product_component_reference import SubscriptionProductComponentReference
from postfinancecheckout.models.subscription_product_component_reference_state import SubscriptionProductComponentReferenceState
from postfinancecheckout.models.subscription_product_component_update import SubscriptionProductComponentUpdate
from postfinancecheckout.models.subscription_product_create import SubscriptionProductCreate
from postfinancecheckout.models.subscription_product_retirement import SubscriptionProductRetirement
from postfinancecheckout.models.subscription_product_retirement_request import SubscriptionProductRetirementRequest
from postfinancecheckout.models.subscription_product_state import SubscriptionProductState
from postfinancecheckout.models.subscription_product_version import SubscriptionProductVersion
from postfinancecheckout.models.subscription_product_version_pending import SubscriptionProductVersionPending
from postfinancecheckout.models.subscription_product_version_retirement import SubscriptionProductVersionRetirement
from postfinancecheckout.models.subscription_product_version_retirement_request import SubscriptionProductVersionRetirementRequest
from postfinancecheckout.models.subscription_product_version_state import SubscriptionProductVersionState
from postfinancecheckout.models.subscription_search_response import SubscriptionSearchResponse
from postfinancecheckout.models.subscription_state import SubscriptionState
from postfinancecheckout.models.subscription_suspension import SubscriptionSuspension
from postfinancecheckout.models.subscription_suspension_action import SubscriptionSuspensionAction
from postfinancecheckout.models.subscription_suspension_reason import SubscriptionSuspensionReason
from postfinancecheckout.models.subscription_suspension_state import SubscriptionSuspensionState
from postfinancecheckout.models.subscription_update import SubscriptionUpdate
from postfinancecheckout.models.subscription_update_request import SubscriptionUpdateRequest
from postfinancecheckout.models.subscription_version import SubscriptionVersion
from postfinancecheckout.models.subscription_version_list_response import SubscriptionVersionListResponse
from postfinancecheckout.models.subscription_version_search_response import SubscriptionVersionSearchResponse
from postfinancecheckout.models.subscription_version_state import SubscriptionVersionState
from postfinancecheckout.models.suspension_creation_request import SuspensionCreationRequest
from postfinancecheckout.models.suspension_list_response import SuspensionListResponse
from postfinancecheckout.models.suspension_search_response import SuspensionSearchResponse
from postfinancecheckout.models.tax import Tax
from postfinancecheckout.models.tax_calculation import TaxCalculation
from postfinancecheckout.models.tax_class import TaxClass
from postfinancecheckout.models.tax_create import TaxCreate
from postfinancecheckout.models.tenant_database import TenantDatabase
from postfinancecheckout.models.terminal_list_response import TerminalListResponse
from postfinancecheckout.models.terminal_receipt_format import TerminalReceiptFormat
from postfinancecheckout.models.terminal_search_response import TerminalSearchResponse
from postfinancecheckout.models.terminal_transaction_summary_list_response import TerminalTransactionSummaryListResponse
from postfinancecheckout.models.terminal_transaction_summary_search_response import TerminalTransactionSummarySearchResponse
from postfinancecheckout.models.token import Token
from postfinancecheckout.models.token_create import TokenCreate
from postfinancecheckout.models.token_list_response import TokenListResponse
from postfinancecheckout.models.token_search_response import TokenSearchResponse
from postfinancecheckout.models.token_update import TokenUpdate
from postfinancecheckout.models.token_version import TokenVersion
from postfinancecheckout.models.token_version_list_response import TokenVersionListResponse
from postfinancecheckout.models.token_version_retry_strategy import TokenVersionRetryStrategy
from postfinancecheckout.models.token_version_search_response import TokenVersionSearchResponse
from postfinancecheckout.models.token_version_state import TokenVersionState
from postfinancecheckout.models.token_version_type import TokenVersionType
from postfinancecheckout.models.tokenization_mode import TokenizationMode
from postfinancecheckout.models.tokenized_card_data import TokenizedCardData
from postfinancecheckout.models.tokenized_card_data_create import TokenizedCardDataCreate
from postfinancecheckout.models.tokenized_card_request import TokenizedCardRequest
from postfinancecheckout.models.transaction import Transaction
from postfinancecheckout.models.transaction_client_platform_information import TransactionClientPlatformInformation
from postfinancecheckout.models.transaction_comment import TransactionComment
from postfinancecheckout.models.transaction_comment_active import TransactionCommentActive
from postfinancecheckout.models.transaction_comment_create import TransactionCommentCreate
from postfinancecheckout.models.transaction_comment_list_response import TransactionCommentListResponse
from postfinancecheckout.models.transaction_comment_search_response import TransactionCommentSearchResponse
from postfinancecheckout.models.transaction_completion import TransactionCompletion
from postfinancecheckout.models.transaction_completion_behavior import TransactionCompletionBehavior
from postfinancecheckout.models.transaction_completion_details import TransactionCompletionDetails
from postfinancecheckout.models.transaction_completion_mode import TransactionCompletionMode
from postfinancecheckout.models.transaction_completion_state import TransactionCompletionState
from postfinancecheckout.models.transaction_create import TransactionCreate
from postfinancecheckout.models.transaction_environment_selection_strategy import TransactionEnvironmentSelectionStrategy
from postfinancecheckout.models.transaction_group import TransactionGroup
from postfinancecheckout.models.transaction_group_state import TransactionGroupState
from postfinancecheckout.models.transaction_invoice import TransactionInvoice
from postfinancecheckout.models.transaction_invoice_comment import TransactionInvoiceComment
from postfinancecheckout.models.transaction_invoice_comment_active import TransactionInvoiceCommentActive
from postfinancecheckout.models.transaction_invoice_comment_create import TransactionInvoiceCommentCreate
from postfinancecheckout.models.transaction_invoice_replacement import TransactionInvoiceReplacement
from postfinancecheckout.models.transaction_invoice_state import TransactionInvoiceState
from postfinancecheckout.models.transaction_line_item_version import TransactionLineItemVersion
from postfinancecheckout.models.transaction_line_item_version_create import TransactionLineItemVersionCreate
from postfinancecheckout.models.transaction_line_item_version_state import TransactionLineItemVersionState
from postfinancecheckout.models.transaction_list_response import TransactionListResponse
from postfinancecheckout.models.transaction_pending import TransactionPending
from postfinancecheckout.models.transaction_search_response import TransactionSearchResponse
from postfinancecheckout.models.transaction_state import TransactionState
from postfinancecheckout.models.transaction_user_interface_type import TransactionUserInterfaceType
from postfinancecheckout.models.transaction_void import TransactionVoid
from postfinancecheckout.models.transaction_void_list_response import TransactionVoidListResponse
from postfinancecheckout.models.transaction_void_mode import TransactionVoidMode
from postfinancecheckout.models.transaction_void_search_response import TransactionVoidSearchResponse
from postfinancecheckout.models.transaction_void_state import TransactionVoidState
from postfinancecheckout.models.two_factor_authentication_type import TwoFactorAuthenticationType
from postfinancecheckout.models.user import User
from postfinancecheckout.models.user_account_role import UserAccountRole
from postfinancecheckout.models.user_account_role_list_response import UserAccountRoleListResponse
from postfinancecheckout.models.user_space_role import UserSpaceRole
from postfinancecheckout.models.user_space_role_list_response import UserSpaceRoleListResponse
from postfinancecheckout.models.user_type import UserType
from postfinancecheckout.models.wallet_type import WalletType
from postfinancecheckout.models.web_app_confirmation_response import WebAppConfirmationResponse
from postfinancecheckout.models.webhook_identity import WebhookIdentity
from postfinancecheckout.models.webhook_listener import WebhookListener
from postfinancecheckout.models.webhook_listener_create import WebhookListenerCreate
from postfinancecheckout.models.webhook_listener_entity import WebhookListenerEntity
from postfinancecheckout.models.webhook_listener_list_response import WebhookListenerListResponse
from postfinancecheckout.models.webhook_listener_search_response import WebhookListenerSearchResponse
from postfinancecheckout.models.webhook_listener_update import WebhookListenerUpdate
from postfinancecheckout.models.webhook_url_list_response import WebhookURLListResponse
from postfinancecheckout.models.webhook_url_search_response import WebhookURLSearchResponse
from postfinancecheckout.models.webhook_url import WebhookUrl
from postfinancecheckout.models.webhook_url_create import WebhookUrlCreate
from postfinancecheckout.models.webhook_url_update import WebhookUrlUpdate
